@@ -23,10 +23,7 @@ import numbers
 
 from six.moves import xrange  # pylint: disable=redefined-builtin
 
-# pylint: disable=wildcard-import,undefined-variable
-# pylint: enable=wildcard-import
-
-from tensorflow.contrib.factorization.python.ops.gen_factorization_ops import *
+from tensorflow.contrib.factorization.python.ops import gen_factorization_ops
 from tensorflow.contrib.util import loader
 from tensorflow.python.framework import constant_op
 from tensorflow.python.framework import dtypes
@@ -42,6 +39,7 @@ from tensorflow.python.ops import math_ops
 from tensorflow.python.ops import random_ops
 from tensorflow.python.ops import sparse_ops
 from tensorflow.python.ops import state_ops
+from tensorflow.python.ops import variable_scope
 from tensorflow.python.ops import variables
 from tensorflow.python.platform import resource_loader
 
@@ -53,19 +51,22 @@ class WALSModel(object):
   r"""A model for Weighted Alternating Least Squares matrix factorization.
 
   It minimizes the following loss function over U, V:
-   \\( ||W \odot (A - U V^T) ||_F^2 + \lambda (||U||_F^2 + ||V||_F^2) )\\
+  $$
+   \|\sqrt W \odot (A - U V^T)\|_F^2 + \lambda (\|U\|_F^2 + \|V\|_F^2)
+  $$
     where,
     A: input matrix,
-    W: weight matrix,
+    W: weight matrix. Note that the (element-wise) square root of the weights
+      is used in the objective function.
     U, V: row_factors and column_factors matrices,
     \\(\lambda)\\: regularization.
   Also we assume that W is of the following special form:
-  \\( W_{ij} = W_0 + R_i * C_j )\\  if \\(A_{ij} \ne 0)\\,
-  \\(W_{ij} = W_0)\\ otherwise.
+  \\( W_{ij} = W_0 + R_i * C_j \\)  if \\(A_{ij} \ne 0\\),
+  \\(W_{ij} = W_0\\) otherwise.
   where,
-  \\(W_0)\\: unobserved_weight,
-  \\(R_i)\\: row_weights,
-  \\(C_j)\\: col_weights.
+  \\(W_0\\): unobserved_weight,
+  \\(R_i\\): row_weights,
+  \\(C_j\\): col_weights.
 
   Note that the current implementation supports two operation modes: The default
   mode is for the condition where row_factors and col_factors can individually
@@ -75,6 +76,22 @@ class WALSModel(object):
   creating the worker caches and instead the relevant weight and factor values
   are looked up from parameter servers at each step.
 
+  Loss computation: The loss can be computed efficiently by decomposing it into
+  a sparse term and a Gramian term, see wals.md.
+  The loss is returned by the update_{col, row}_factors(sp_input), and is
+  normalized as follows:
+    _, _, unregularized_loss, regularization, sum_weights =
+        update_row_factors(sp_input)
+  if sp_input contains the rows \\({A_i, i \in I}\\), and the input matrix A
+  has n total rows, then the minibatch loss = unregularized_loss +
+  regularization is
+   $$
+   (\|\sqrt W_I \odot (A_I - U_I V^T)\|_F^2 + \lambda \|U_I\|_F^2) * n / |I| +
+   \lambda \|V\|_F^2
+   $$
+  The sum_weights tensor contains the normalized sum of weights
+  \\(sum(W_I) * n / |I|\\).
+
   A typical usage example (pseudocode):
 
     with tf.Graph().as_default():
@@ -87,35 +104,37 @@ class WALSModel(object):
       model_init_op = model.initialize_op
 
       # To be run once per worker after session is available, prior to
-      # the gramian_prep_ops for row(column) can be run.
+      # the prep_gramian_op for row(column) can be run.
       worker_init_op = model.worker_init
 
-      # To be run once per interation sweep before the row(column) update
+      # To be run once per iteration sweep before the row(column) update
       # initialize ops can be run. Note that in the distributed training
       # situations, this should only be run by the chief trainer. All other
       # trainers need to block until this is done.
-      row_update_gramian_prep_op = model.row_update_prep_gramian_op
-      col_update_gramian_prep_op = model.col_update_prep_gramian_op
+      row_update_prep_gramian_op = model.row_update_prep_gramian_op
+      col_update_prep_gramian_op = model.col_update_prep_gramian_op
 
       # To be run once per worker per iteration sweep. Must be run before
       # any actual update ops can be run.
       init_row_update_op = model.initialize_row_update_op
       init_col_update_op = model.initialize_col_update_op
 
-      # Ops to upate row(column). This can either take the entire sparse tensor
-      # or slices of sparse tensor. For distributed trainer, each trainer
-      # handles just part of the matrix.
-      row_update_op = model.update_row_factors(
-           sp_input=matrix_slices_from_queue_for_worker_shard)[1]
-      col_update_op = model.update_col_factors(
+      # Ops to update row(column). This can either take the entire sparse
+      # tensor or slices of sparse tensor. For distributed trainer, each
+      # trainer handles just part of the matrix.
+      _, row_update_op, unreg_row_loss, row_reg, _ = model.update_row_factors(
+           sp_input=matrix_slices_from_queue_for_worker_shard)
+      row_loss = unreg_row_loss + row_reg
+      _, col_update_op, unreg_col_loss, col_reg, _ = model.update_col_factors(
            sp_input=transposed_matrix_slices_from_queue_for_worker_shard,
-           transpose_input=True)[1]
+           transpose_input=True)
+      col_loss = unreg_col_loss + col_reg
 
       ...
 
       # model_init_op is passed to Supervisor. Chief trainer runs it. Other
       # trainers wait.
-      sv = tf.Supervisor(is_chief=is_chief,
+      sv = tf.train.Supervisor(is_chief=is_chief,
                          ...,
                          init_op=tf.group(..., model_init_op, ...), ...)
       ...
@@ -134,7 +153,7 @@ class WALSModel(object):
 
           # Row update sweep.
           if is_chief:
-            row_update_gramian_prep_op.run(session=sess)
+            row_update_prep_gramian_op.run(session=sess)
           else:
             wait_for_chief
 
@@ -152,7 +171,7 @@ class WALSModel(object):
 
           # Column update sweep.
           if is_chief:
-            col_update_gramian_prep_op.run(session=sess)
+            col_update_prep_gramian_op.run(session=sess)
           else:
             wait_for_chief
 
@@ -177,7 +196,9 @@ class WALSModel(object):
                num_col_shards=1,
                row_weights=1,
                col_weights=1,
-               use_factors_weights_cache=True):
+               use_factors_weights_cache=True,
+               use_gramian_cache=True,
+               use_scoped_vars=False):
     """Creates model for WALS matrix factorization.
 
     Args:
@@ -201,16 +222,26 @@ class WALSModel(object):
         in the form of [[w_0, w_1, ...], [w_k, ... ], [...]], with the number of
         inner lists matching the number of row factor shards and the elements in
         each inner list are the weights for the rows of the corresponding row
-        factor shard. In this case,  w_ij = unonbserved_weight +
+        factor shard. In this case,  w_ij = unobserved_weight +
                                             row_weights[i] * col_weights[j].
         - If this is a single non-negative real number, this value is used for
-        all row weights and w_ij = unobserved_weight + row_weights *
+        all row weights and \\(w_ij\\) = unobserved_weight + row_weights *
                                    col_weights[j].
         Note that it is allowed to have row_weights as a list while col_weights
         a single number or vice versa.
       col_weights: See row_weights.
       use_factors_weights_cache: When True, the factors and weights will be
-        cached on the workers before the updates start. Defaults to True.
+        cached on the workers before the updates start. Defaults to True. Note
+        that the weights cache is initialized through `worker_init`, and the
+        row/col factors cache is initialized through
+        `initialize_{col/row}_update_op`. In the case where the weights are
+        computed outside and set before the training iterations start, it is
+        important to ensure the `worker_init` op is run afterwards for the
+        weights cache to take effect.
+      use_gramian_cache: When True, the Gramians will be cached on the workers
+        before the updates start. Defaults to True.
+      use_scoped_vars: When True, the factor and weight vars will also be nested
+        in a tf.name_scope.
     """
     self._input_rows = input_rows
     self._input_cols = input_cols
@@ -218,33 +249,51 @@ class WALSModel(object):
     self._num_col_shards = num_col_shards
     self._n_components = n_components
     self._unobserved_weight = unobserved_weight
-    self._regularization = (array_ops.diag(
-        constant_op.constant(
-            regularization, shape=[self._n_components], dtype=dtypes.float32))
-                            if regularization is not None else None)
+    self._regularization = regularization
+    self._regularization_matrix = (
+        regularization * linalg_ops.eye(self._n_components)
+        if regularization is not None else None)
     assert (row_weights is None) == (col_weights is None)
-    self._row_weights = WALSModel._create_weights(row_weights, self._input_rows,
-                                                  self._num_row_shards,
-                                                  "row_weights")
-    self._col_weights = WALSModel._create_weights(col_weights, self._input_cols,
-                                                  self._num_col_shards,
-                                                  "col_weights")
     self._use_factors_weights_cache = use_factors_weights_cache
-    self._row_factors = self._create_factors(self._input_rows,
-                                             self._n_components,
-                                             self._num_row_shards, row_init,
-                                             "row_factors")
-    self._col_factors = self._create_factors(self._input_cols,
-                                             self._n_components,
-                                             self._num_col_shards, col_init,
-                                             "col_factors")
+    self._use_gramian_cache = use_gramian_cache
+
+    if use_scoped_vars:
+      with ops.name_scope("row_weights"):
+        self._row_weights = WALSModel._create_weights(
+            row_weights, self._input_rows, self._num_row_shards, "row_weights")
+      with ops.name_scope("col_weights"):
+        self._col_weights = WALSModel._create_weights(
+            col_weights, self._input_cols, self._num_col_shards, "col_weights")
+      with ops.name_scope("row_factors"):
+        self._row_factors = self._create_factors(
+            self._input_rows, self._n_components, self._num_row_shards,
+            row_init, "row_factors")
+      with ops.name_scope("col_factors"):
+        self._col_factors = self._create_factors(
+            self._input_cols, self._n_components, self._num_col_shards,
+            col_init, "col_factors")
+    else:
+      self._row_weights = WALSModel._create_weights(
+          row_weights, self._input_rows, self._num_row_shards, "row_weights")
+      self._col_weights = WALSModel._create_weights(
+          col_weights, self._input_cols, self._num_col_shards, "col_weights")
+      self._row_factors = self._create_factors(
+          self._input_rows, self._n_components, self._num_row_shards, row_init,
+          "row_factors")
+      self._col_factors = self._create_factors(
+          self._input_cols, self._n_components, self._num_col_shards, col_init,
+          "col_factors")
+
     self._row_gramian = self._create_gramian(self._n_components, "row_gramian")
     self._col_gramian = self._create_gramian(self._n_components, "col_gramian")
-    self._row_update_prep_gramian = self._prepare_gramian(self._col_factors,
-                                                          self._col_gramian)
-    self._col_update_prep_gramian = self._prepare_gramian(self._row_factors,
-                                                          self._row_gramian)
-    self._create_transient_vars()
+    with ops.name_scope("row_prepare_gramian"):
+      self._row_update_prep_gramian = self._prepare_gramian(
+          self._col_factors, self._col_gramian)
+    with ops.name_scope("col_prepare_gramian"):
+      self._col_update_prep_gramian = self._prepare_gramian(
+          self._row_factors, self._row_gramian)
+    with ops.name_scope("transient_vars"):
+      self._create_transient_vars()
 
   @property
   def row_factors(self):
@@ -311,7 +360,7 @@ class WALSModel(object):
       var_name = "%s_shard_%d" % (name, i)
       var_init = make_initializer(i, size)
       sharded_matrix.append(
-          variables.Variable(
+          variable_scope.variable(
               var_init, dtype=dtypes.float32, name=var_name))
 
     return sharded_matrix
@@ -370,7 +419,7 @@ class WALSModel(object):
       var_name = "%s_shard_%d" % (name, i)
       var_init = make_wt_initializer(i, size)
       sharded_weight.append(
-          variables.Variable(
+          variable_scope.variable(
               var_init, dtype=dtypes.float32, name=var_name))
 
     return sharded_weight
@@ -387,7 +436,7 @@ class WALSModel(object):
     Returns:
       A gramian Tensor with shape of [n_components, n_components].
     """
-    return variables.Variable(
+    return variable_scope.variable(
         array_ops.zeros([n_components, n_components]),
         dtype=dtypes.float32,
         name=name)
@@ -395,7 +444,7 @@ class WALSModel(object):
   @staticmethod
   def _transient_var(name):
     """Helper function to create a Variable."""
-    return variables.Variable(
+    return variable_scope.variable(
         1.0,
         trainable=False,
         collections=[ops.GraphKeys.LOCAL_VARIABLES],
@@ -411,7 +460,7 @@ class WALSModel(object):
       gramian: Variable storing the gramian calculated from the factors.
 
     Returns:
-      A op that updates the gramian with the calcuated value from the factors.
+      An op that updates the gramian with the calculated value from the factors.
     """
     partial_gramians = []
     for f in factors:
@@ -482,10 +531,13 @@ class WALSModel(object):
     """Creates local cache of factors, weights and gramian for rows and columns.
 
     Note that currently the caching strategy is as follows:
-    When initiating a row(column) update, the column(row) gramian is computed
-    and cached while the row gramian is reset; optionally, column(row) factors
-    and weights are cached and row(column) factors and weights are reset when
-    use_factors_weights_cache is True.
+    When initiating a row (resp. column) update:
+      - The column (resp. row) gramian is computed.
+      - Optionally, if use_gramian_cache is True, the column (resp. row) Gramian
+        is cached, while the row (resp. column) gramian is reset.
+      - Optionally, if use_factors_weights_cache is True, the column (resp. row)
+        factors and weights are cached, while the row (resp. column) factors and
+        weights are reset.
     """
 
     (self._row_factors_cache, row_factors_cache_init,
@@ -502,27 +554,27 @@ class WALSModel(object):
         self._row_weights,
         "row_wt_cache",
         pass_through=not self._use_factors_weights_cache)
-
     (self._col_wt_cache, col_wt_cache_init, _) = self._cached_copy(
         self._col_weights,
         "col_wt_cache",
         pass_through=not self._use_factors_weights_cache)
-
     (self._row_gramian_cache, row_gramian_cache_init,
      row_gramian_cache_reset) = self._cached_copy(
-         self._row_gramian, "row_gramian_cache", pass_through=False)
+         self._row_gramian,
+         "row_gramian_cache",
+         pass_through=not self._use_gramian_cache)
     (self._col_gramian_cache, col_gramian_cache_init,
      col_gramian_cache_reset) = self._cached_copy(
-         self._col_gramian, "col_gramian_cache", pass_through=False)
+         self._col_gramian,
+         "col_gramian_cache",
+         pass_through=not self._use_gramian_cache)
 
-    self._row_updates_init = control_flow_ops.group(col_factors_cache_init,
-                                                    row_factors_cache_reset,
-                                                    col_gramian_cache_init,
-                                                    row_gramian_cache_reset)
-    self._col_updates_init = control_flow_ops.group(row_factors_cache_init,
-                                                    col_factors_cache_reset,
-                                                    row_gramian_cache_init,
-                                                    col_gramian_cache_reset)
+    self._row_updates_init = control_flow_ops.group(
+        col_factors_cache_init, row_factors_cache_reset, col_gramian_cache_init,
+        row_gramian_cache_reset)
+    self._col_updates_init = control_flow_ops.group(
+        row_factors_cache_init, col_factors_cache_reset, row_gramian_cache_init,
+        col_gramian_cache_reset)
 
     if self._row_wt_cache is not None:
       assert self._col_wt_cache is not None
@@ -533,7 +585,14 @@ class WALSModel(object):
 
   @property
   def worker_init(self):
-    """Op to initialize worker state once before starting any updates."""
+    """Op to initialize worker state once before starting any updates.
+
+    Note that specifically this initializes the cache of the row and column
+    weights on workers when `use_factors_weights_cache` is True. In this case,
+    if these weights are being calculated and reset after the object is created,
+    it is important to ensure this ops is run afterwards so the cache reflects
+    the correct values.
+    """
     return self._worker_init
 
   @property
@@ -542,6 +601,9 @@ class WALSModel(object):
 
     Must be run before initialize_row_update_op and should only be run by one
     trainer (usually the chief) when doing distributed training.
+
+    Returns:
+      Op to form the gramian.
     """
     return self._row_update_prep_gramian
 
@@ -551,6 +613,9 @@ class WALSModel(object):
 
     Must be run before initialize_col_update_op and should only be run by one
     trainer (usually the chief) when doing distributed training.
+
+    Returns:
+      Op to form the gramian.
     """
     return self._col_update_prep_gramian
 
@@ -584,13 +649,14 @@ class WALSModel(object):
     return func
 
   @classmethod
-  def scatter_update(cls, factor, indices, values, sharding_func):
+  def scatter_update(cls, factor, indices, values, sharding_func, name=None):
     """Helper function for doing sharded scatter update."""
     assert isinstance(factor, list)
     if len(factor) == 1:
       with ops.colocate_with(factor[0]):
         # TODO(agarwal): assign instead of scatter update for full batch update.
-        return state_ops.scatter_update(factor[0], indices, values).op
+        return state_ops.scatter_update(
+            factor[0], indices, values, name=name).op
     else:
       num_shards = len(factor)
       assignments, new_ids = sharding_func(indices)
@@ -605,10 +671,10 @@ class WALSModel(object):
         updates.append(
             state_ops.scatter_update(factor[i], sharded_ids[i], sharded_values[
                 i]))
-      return control_flow_ops.group(*updates)
+      return control_flow_ops.group(*updates, name=name)
 
   def update_row_factors(self, sp_input=None, transpose_input=False):
-    """Updates the row factors.
+    r"""Updates the row factors.
 
     Args:
       sp_input: A SparseTensor representing a subset of rows of the full input
@@ -618,16 +684,30 @@ class WALSModel(object):
         rows corresponding to the transposed input are updated.
 
     Returns:
-      A tuple consisting of the following two elements:
+      A tuple consisting of the following elements:
       new_values: New values for the row factors.
       update_op: An op that assigns the newly computed values to the row
         factors.
+      unregularized_loss: A tensor (scalar) that contains the normalized
+        minibatch loss corresponding to sp_input, without the regularization
+        term. If sp_input contains the rows \\({A_{i, :}, i \in I}\\), and the
+        input matrix A has n total rows, then the unregularized loss is:
+        \\(\|\sqrt W_I \odot (A_I - U_I V^T)\|_F^2 * n / |I|\\)
+        The total loss is unregularized_loss + regularization.
+      regularization: A tensor (scalar) that contains the normalized
+        regularization term for the minibatch loss corresponding to sp_input.
+        If sp_input contains the rows \\({A_{i, :}, i \in I}\\), and the input
+        matrix A has n total rows, then the regularization term is:
+        \\(\lambda \|U_I\|_F^2) * n / |I| + \lambda \|V\|_F^2\\).
+      sum_weights: The sum of the weights W_I corresponding to sp_input,
+        normalized by a factor of \\(n / |I|\\). The root weighted squared
+        error is: \sqrt(unregularized_loss / sum_weights).
     """
     return self._process_input_helper(
         True, sp_input=sp_input, transpose_input=transpose_input)
 
   def update_col_factors(self, sp_input=None, transpose_input=False):
-    """Updates the column factors.
+    r"""Updates the column factors.
 
     Args:
       sp_input: A SparseTensor representing a subset of columns of the full
@@ -637,10 +717,24 @@ class WALSModel(object):
         columns corresponding to the transposed input are updated.
 
     Returns:
-      A tuple consisting of the following two elements:
+      A tuple consisting of the following elements:
       new_values: New values for the column factors.
       update_op: An op that assigns the newly computed values to the column
         factors.
+      unregularized_loss: A tensor (scalar) that contains the normalized
+        minibatch loss corresponding to sp_input, without the regularization
+        term. If sp_input contains the columns \\({A_{:, j}, j \in J}\\), and
+        the input matrix A has m total columns, then the unregularized loss is:
+        \\(\|\sqrt W_J \odot (A_J - U V_J^T)\|_F^2 * m / |I|\\)
+        The total loss is unregularized_loss + regularization.
+      regularization: A tensor (scalar) that contains the normalized
+        regularization term for the minibatch loss corresponding to sp_input.
+        If sp_input contains the columns \\({A_{:, j}, j \in J}\\), and the
+        input matrix A has m total columns, then the regularization term is:
+        \\(\lambda \|V_J\|_F^2) * m / |J| + \lambda \|U\|_F^2\\).
+      sum_weights: The sum of the weights W_J corresponding to sp_input,
+        normalized by a factor of \\(m / |J|\\). The root weighted squared
+        error is: \sqrt(unregularized_loss / sum_weights).
     """
     return self._process_input_helper(
         False, sp_input=sp_input, transpose_input=transpose_input)
@@ -651,8 +745,8 @@ class WALSModel(object):
                           projection_weights=None):
     """Projects the row factors.
 
-    This computes the row embedding u_i for an observed row a_i by solving
-    one iteration of the update equations.
+    This computes the row embedding \\(u_i\\) for an observed row \\(a_i\\) by
+    solving one iteration of the update equations.
 
     Args:
       sp_input: A SparseTensor representing a set of rows. Please note that the
@@ -684,8 +778,8 @@ class WALSModel(object):
                           projection_weights=None):
     """Projects the column factors.
 
-    This computes the column embedding v_j for an observed column a_j by solving
-    one iteration of the update equations.
+    This computes the column embedding \\(v_j\\) for an observed column
+    \\(a_j\\) by solving one iteration of the update equations.
 
     Args:
       sp_input: A SparseTensor representing a set of columns. Please note that
@@ -734,10 +828,18 @@ class WALSModel(object):
         of columns to be updated/projected.
 
     Returns:
-      A tuple consisting of the following two elements:
+      A tuple consisting of the following elements:
       new_values: New values for the row/column factors.
       update_op: An op that assigns the newly computed values to the row/column
         factors.
+      unregularized_loss: A tensor (scalar) that contains the normalized
+        minibatch loss corresponding to sp_input, without the regularization
+        term. Add the regularization term below to yield the loss.
+      regularization: A tensor (scalar) that contains the normalized
+        regularization term for the minibatch loss corresponding to sp_input.
+      sum_weights: The sum of the weights corresponding to sp_input. This
+        can be used with unregularized loss to calculate the root weighted
+        squared error.
     """
     assert isinstance(sp_input, sparse_tensor.SparseTensor)
 
@@ -746,6 +848,8 @@ class WALSModel(object):
       right_factors = self._col_factors_cache
       row_wt = self._row_wt_cache
       col_wt = self._col_wt_cache
+      total_rows = self._input_rows
+      total_cols = self._input_cols
       sharding_func = WALSModel._get_sharding_func(self._input_rows,
                                                    self._num_row_shards)
       gramian = self._col_gramian_cache
@@ -754,6 +858,8 @@ class WALSModel(object):
       right_factors = self._row_factors_cache
       row_wt = self._col_wt_cache
       col_wt = self._row_wt_cache
+      total_rows = self._input_cols
+      total_cols = self._input_rows
       sharding_func = WALSModel._get_sharding_func(self._input_cols,
                                                    self._num_col_shards)
       gramian = self._row_gramian_cache
@@ -789,8 +895,8 @@ class WALSModel(object):
     right = embedding_ops.embedding_lookup(
         right_factors, gather_indices, partition_strategy="div")
     new_sp_indices = array_ops.concat([row_ids, col_ids], 1)
-    new_sp_shape = (array_ops.concat([row_shape, col_shape], 0) if
-                    transpose_input else
+    new_sp_shape = (array_ops.concat([row_shape, col_shape], 0)
+                    if transpose_input else
                     array_ops.concat([col_shape, row_shape], 0))
     new_sp_input = sparse_tensor.SparseTensor(
         indices=new_sp_indices,
@@ -799,13 +905,13 @@ class WALSModel(object):
 
     # Compute lhs and rhs of the normal equations
     total_lhs = (self._unobserved_weight * gramian)
-    if self._regularization is not None:
-      total_lhs += self._regularization
+    if self._regularization_matrix is not None:
+      total_lhs += self._regularization_matrix
     if self._row_weights is None:
       # Special case of ALS. Use a much simpler update rule.
-      total_rhs = (self._unobserved_weight *
-                   sparse_ops.sparse_tensor_dense_matmul(
-                       new_sp_input, right, adjoint_a=transpose_input))
+      total_rhs = (
+          self._unobserved_weight * sparse_ops.sparse_tensor_dense_matmul(
+              new_sp_input, right, adjoint_a=transpose_input))
       # TODO(rmlarsen): handle transposing in tf.matrix_solve instead of
       # transposing explicitly.
       # TODO(rmlarsen): multi-thread tf.matrix_solve.
@@ -819,30 +925,92 @@ class WALSModel(object):
         row_weights_slice = embedding_ops.embedding_lookup(
             row_wt, update_indices, partition_strategy="div")
       else:
+        num_indices = array_ops.shape(update_indices)[0]
         with ops.control_dependencies(
             [check_ops.assert_less_equal(array_ops.rank(row_weights), 1)]):
           row_weights_slice = control_flow_ops.cond(
               math_ops.equal(array_ops.rank(row_weights), 0),
-              lambda: (array_ops.ones([array_ops.shape(update_indices)[0]]) * row_weights),
+              lambda: (array_ops.ones([num_indices]) * row_weights),
               lambda: math_ops.cast(row_weights, dtypes.float32))
 
       col_weights = embedding_ops.embedding_lookup(
           col_wt, gather_indices, partition_strategy="div")
-      partial_lhs, total_rhs = wals_compute_partial_lhs_and_rhs(
-          right,
-          col_weights,
-          self._unobserved_weight,
-          row_weights_slice,
-          new_sp_input.indices,
-          new_sp_input.values,
-          num_rows,
-          transpose_input,
-          name="wals_compute_partial_lhs_rhs")
+      partial_lhs, total_rhs = (
+          gen_factorization_ops.wals_compute_partial_lhs_and_rhs(
+              right,
+              col_weights,
+              self._unobserved_weight,
+              row_weights_slice,
+              new_sp_input.indices,
+              new_sp_input.values,
+              num_rows,
+              transpose_input,
+              name="wals_compute_partial_lhs_rhs"))
       total_lhs = array_ops.expand_dims(total_lhs, 0) + partial_lhs
       total_rhs = array_ops.expand_dims(total_rhs, -1)
       new_left_values = array_ops.squeeze(
           linalg_ops.matrix_solve(total_lhs, total_rhs), [2])
 
-    return (new_left_values, self.scatter_update(left, update_indices,
-                                                 new_left_values,
-                                                 sharding_func))
+    update_op_name = "row_update" if update_row_factors else "col_update"
+    update_op = self.scatter_update(
+        left,
+        update_indices,
+        new_left_values,
+        sharding_func,
+        name=update_op_name)
+
+    # Create the loss subgraph
+    loss_sp_input = (sparse_ops.sparse_transpose(new_sp_input)
+                     if transpose_input else new_sp_input)
+    # sp_approx is the low rank estimate of the input matrix, formed by
+    # computing the product <\\(u_i, v_j\\)> for (i, j) in loss_sp_input.indices.
+    sp_approx_vals = gen_factorization_ops.masked_matmul(
+        new_left_values,
+        right,
+        loss_sp_input.indices,
+        transpose_a=False,
+        transpose_b=True)
+    sp_approx = sparse_tensor.SparseTensor(
+        loss_sp_input.indices, sp_approx_vals, loss_sp_input.dense_shape)
+    sp_approx_sq = math_ops.square(sp_approx)
+    sp_residual = sparse_ops.sparse_add(loss_sp_input, sp_approx * (-1))
+    sp_residual_sq = math_ops.square(sp_residual)
+    row_wt_mat = (constant_op.constant(0.)
+                  if self._row_weights is None else array_ops.expand_dims(
+                      row_weights_slice, 1))
+    col_wt_mat = (constant_op.constant(0.)
+                  if self._col_weights is None else array_ops.expand_dims(
+                      col_weights, 0))
+
+    # We return the normalized loss
+    partial_row_gramian = math_ops.matmul(
+        new_left_values, new_left_values, transpose_a=True)
+    normalization_factor = total_rows / math_ops.cast(num_rows, dtypes.float32)
+
+    unregularized_loss = (
+        self._unobserved_weight * (  # pyformat line break
+            sparse_ops.sparse_reduce_sum(sp_residual_sq) -  # pyformat break
+            sparse_ops.sparse_reduce_sum(sp_approx_sq) +  # pyformat break
+            math_ops.trace(math_ops.matmul(partial_row_gramian, gramian))) +
+        sparse_ops.sparse_reduce_sum(row_wt_mat * (sp_residual_sq * col_wt_mat))
+    ) * normalization_factor
+
+    if self._regularization is not None:
+      regularization = self._regularization * (
+          math_ops.trace(partial_row_gramian) * normalization_factor +
+          math_ops.trace(gramian))
+    else:
+      regularization = constant_op.constant(0.)
+
+    sum_weights = self._unobserved_weight * math_ops.cast(
+        total_rows * total_cols, dtypes.float32)
+    if self._row_weights is not None and self._col_weights is not None:
+      ones = sparse_tensor.SparseTensor(
+          indices=loss_sp_input.indices,
+          values=array_ops.ones(array_ops.shape(loss_sp_input.values)),
+          dense_shape=loss_sp_input.dense_shape)
+      sum_weights += sparse_ops.sparse_reduce_sum(row_wt_mat * (
+          ones * col_wt_mat)) * normalization_factor
+
+    return (new_left_values, update_op, unregularized_loss, regularization,
+            sum_weights)
