@@ -67,8 +67,27 @@ flags = flags_lib
 FLAGS = flags.FLAGS
 
 
+# Support folding two types of batch norm ops:
+# BatchNormWithGlobalNormalization and FusedBatchNorm.  The two types only
+# differ in input order and attribute names, so we've collected their
+# differences up front.
+INPUT_ORDER = {
+    # Order of inputs for BatchNormWithGlobalNormalization.
+    "BatchNormWithGlobalNormalization": [
+        "conv_op", "mean_op", "var_op", "beta_op", "gamma_op"
+    ],
+    # Order of inputs for FusedBatchNorm.
+    "FusedBatchNorm": ["conv_op", "gamma_op", "beta_op", "mean_op", "var_op"]
+}
+# Name of the attribute epsilon value is stored in.
+EPSILON_ATTR = {
+    "BatchNormWithGlobalNormalization": "variance_epsilon",
+    "FusedBatchNorm": "epsilon"
+}
+
+
 def optimize_for_inference(input_graph_def, input_node_names, output_node_names,
-                           placeholder_type_enum):
+                           placeholder_type_enum, toco_compatible=False):
   """Applies a series of inference optimizations on the input graph.
 
   Args:
@@ -79,20 +98,23 @@ def optimize_for_inference(input_graph_def, input_node_names, output_node_names,
       results.
     placeholder_type_enum: The AttrValue enum for the placeholder data type, or
         a list that specifies one value per input node name.
+    toco_compatible: Boolean, if True, only runs optimizations that result in
+      TOCO compatible graph operations (default=False).
 
   Returns:
     An optimized version of the input graph.
   """
   ensure_graph_is_valid(input_graph_def)
   optimized_graph_def = input_graph_def
-  optimized_graph_def = strip_unused_lib.strip_unused(optimized_graph_def,
-                                                      input_node_names,
-                                                      output_node_names,
-                                                      placeholder_type_enum)
-  optimized_graph_def = graph_util.remove_training_nodes(optimized_graph_def)
+  optimized_graph_def = strip_unused_lib.strip_unused(
+      optimized_graph_def, input_node_names, output_node_names,
+      placeholder_type_enum)
+  optimized_graph_def = graph_util.remove_training_nodes(
+      optimized_graph_def, output_node_names)
   optimized_graph_def = fold_batch_norms(optimized_graph_def)
-  optimized_graph_def = fuse_resize_and_conv(optimized_graph_def,
-                                             output_node_names)
+  if not toco_compatible:
+    optimized_graph_def = fuse_resize_and_conv(optimized_graph_def,
+                                               output_node_names)
   ensure_graph_is_valid(optimized_graph_def)
   return optimized_graph_def
 
@@ -172,6 +194,13 @@ def values_from_const(node_def):
   return tensor_value
 
 
+# Whether to scale by gamma after normalization.
+def scale_after_normalization(node):
+  if node.op == "BatchNormWithGlobalNormalization":
+    return node.attr["scale_after_normalization"].b
+  return True
+
+
 def fold_batch_norms(input_graph_def):
   """Removes batch normalization ops by folding them into convolutions.
 
@@ -194,7 +223,6 @@ def fold_batch_norms(input_graph_def):
   Raises:
     ValueError: If the graph is badly formed with duplicate node names.
   """
-
   input_node_map = {}
   for node in input_graph_def.node:
     if node.name not in input_node_map.keys():
@@ -205,13 +233,14 @@ def fold_batch_norms(input_graph_def):
   nodes_to_skip = {}
   new_ops = []
   for node in input_graph_def.node:
-    if node.op != "BatchNormWithGlobalNormalization":
+    if node.op not in ("BatchNormWithGlobalNormalization", "FusedBatchNorm"):
       continue
 
-    conv_op = node_from_map(input_node_map, node.input[0])
+    conv_op = node_from_map(input_node_map,
+                            node.input[INPUT_ORDER[node.op].index("conv_op")])
     if conv_op.op != "Conv2D":
-      tf_logging.warning("Didn't find expected Conv2D input to '%s'" %
-                         node.name)
+      tf_logging.warning(
+          "Didn't find expected Conv2D input to '%s'" % node.name)
       continue
 
     weights_op = node_from_map(input_node_map, conv_op.input[1])
@@ -223,7 +252,8 @@ def fold_batch_norms(input_graph_def):
     weights = values_from_const(weights_op)
     channel_count = weights.shape[3]
 
-    mean_op = node_from_map(input_node_map, node.input[1])
+    mean_op = node_from_map(input_node_map,
+                            node.input[INPUT_ORDER[node.op].index("mean_op")])
     if mean_op.op != "Const":
       tf_logging.warning("Didn't find expected mean Constant input to '%s',"
                          " found %s instead. Maybe because freeze_graph wasn't"
@@ -236,7 +266,8 @@ def fold_batch_norms(input_graph_def):
                              (channel_count,)), node.name))
       continue
 
-    var_op = node_from_map(input_node_map, node.input[2])
+    var_op = node_from_map(input_node_map,
+                           node.input[INPUT_ORDER[node.op].index("var_op")])
     if var_op.op != "Const":
       tf_logging.warning("Didn't find expected var Constant input to '%s',"
                          " found %s instead. Maybe because freeze_graph wasn't"
@@ -249,7 +280,8 @@ def fold_batch_norms(input_graph_def):
                              (channel_count,)), node.name))
       continue
 
-    beta_op = node_from_map(input_node_map, node.input[3])
+    beta_op = node_from_map(input_node_map,
+                            node.input[INPUT_ORDER[node.op].index("beta_op")])
     if beta_op.op != "Const":
       tf_logging.warning("Didn't find expected beta Constant input to '%s',"
                          " found %s instead. Maybe because freeze_graph wasn't"
@@ -262,7 +294,8 @@ def fold_batch_norms(input_graph_def):
                              (channel_count,)), node.name))
       continue
 
-    gamma_op = node_from_map(input_node_map, node.input[4])
+    gamma_op = node_from_map(input_node_map,
+                             node.input[INPUT_ORDER[node.op].index("gamma_op")])
     if gamma_op.op != "Const":
       tf_logging.warning("Didn't find expected gamma Constant input to '%s',"
                          " found %s instead. Maybe because freeze_graph wasn't"
@@ -275,8 +308,7 @@ def fold_batch_norms(input_graph_def):
                              (channel_count,)), node.name))
       continue
 
-    variance_epsilon_value = node.attr["variance_epsilon"].f
-    scale_after_normalization = node.attr["scale_after_normalization"].b
+    variance_epsilon_value = node.attr[EPSILON_ATTR[node.op]].f
     nodes_to_skip[node.name] = True
     nodes_to_skip[weights_op.name] = True
     nodes_to_skip[mean_op.name] = True
@@ -285,7 +317,7 @@ def fold_batch_norms(input_graph_def):
     nodes_to_skip[gamma_op.name] = True
     nodes_to_skip[conv_op.name] = True
 
-    if scale_after_normalization:
+    if scale_after_normalization(node):
       scale_value = (
           (1.0 / np.vectorize(math.sqrt)(var_value + variance_epsilon_value)) *
           gamma_value)
@@ -320,6 +352,7 @@ def fold_batch_norms(input_graph_def):
     bias_add_op.op = "BiasAdd"
     bias_add_op.name = node.name
     bias_add_op.attr["T"].CopyFrom(conv_op.attr["T"])
+    bias_add_op.attr["data_format"].CopyFrom(conv_op.attr["data_format"])
     bias_add_op.input.extend([new_conv_op.name, offset_op.name])
     new_ops.extend([scaled_weights_op, new_conv_op, offset_op, bias_add_op])
 
@@ -345,6 +378,8 @@ def fuse_resize_and_conv(input_graph_def, output_node_names):
 
   Args:
     input_graph_def: A GraphDef containing a model.
+    output_node_names: A list of names of the nodes that produce the final
+      results.
 
   Returns:
     Modified graph with resize and pad ops merged.
@@ -427,8 +462,8 @@ def fuse_resize_and_conv(input_graph_def, output_node_names):
           resize_op.input[0], resize_op.input[1], mirror_paddings_name,
           conv_op.input[1]
       ])
-      fused_conv_op.attr["resize_align_corners"].CopyFrom(resize_op.attr[
-          "align_corners"])
+      fused_conv_op.attr["resize_align_corners"].CopyFrom(
+          resize_op.attr["align_corners"])
     else:
       fused_conv_op.input.extend(
           [mirror_pad_op.input[0], mirror_paddings_name, conv_op.input[1]])
